<?php

namespace Sudalys\ImportService;

use Sudalys\ImportService\Interfaces\FileProcessorInterface;
use Sudalys\ImportService\Interfaces\ValidatorInterface;
use Sudalys\ImportService\Interfaces\ErrorHandlerInterface;
use Sudalys\ImportService\Interfaces\DataProcessorInterface;
use Sudalys\ImportService\Result\ImportResult;

use function Symfony\Component\DependencyInjection\Loader\Configurator\iterator;

class CsvImporter
{
    private FileProcessorInterface $fileProcessor;
    private ValidatorInterface $validator;
    private ErrorHandlerInterface $errorHandler;
    private DataProcessorInterface $dataProcessor;

    public function __construct(
        FileProcessorInterface $fileProcessor,
        ValidatorInterface $validator,
        ErrorHandlerInterface $errorHandler,
        DataProcessorInterface $dataProcessor
    ) {
        $this->fileProcessor = $fileProcessor;
        $this->validator = $validator;
        $this->errorHandler = $errorHandler;
        $this->dataProcessor = $dataProcessor;
    }

    public function import(string $filePath): ImportResult
    {
        $rows = $this->fileProcessor->processfile($filePath);
        $header = $this->fileProcessor->getHeader($filePath);

        $validationResult = $this->validator->validate($header, $rows);

        if ($validationResult->hasErrors()) {
            $this->errorHandler->handle($validationResult->getErrors());
        }

         $this->dataProcessor->process($rows);
        

        return new ImportResult(
            $validationResult->getErrors(),
            !$validationResult->hasErrors(),
            $validationResult->hasErrors() ? 'Import terminé avec erreurs.' : ' Import réussi.',
            count($rows)
        );
    }
}
